#!/usr/bin/env python
from __future__ import print_function
from past.builtins import basestring
import requests
import json
import re
import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
import cakecms

DEBUG = False

"""

SAMPLE CONFIGURATION ('config.json'):

{
    "lsf": {
        "username": "...",
        "password": "..."
    },
    "exams": [
        {
            "lsf": "https://www.lsf.uni-saarland.de/qisserver/rds?state=notenVerbuchungBearb&...",
            "cms": {
                "url": "...",
                "token": "...",
                "course": "...",
                "registration_id": 1234
            }
        }
    ]
}

"""


def url_to_params(url):
	return {
		'semester': re.search(r'semester%3D(\d+)', url).group(1),
		'termin': re.search(r'termin%3D(\d+)', url).group(1),
		'porgnr': re.search(r'porgnr%3D(\d+)', url).group(1),
		'prueferID': re.search(r'prueferID%3D([a-z]?\d+)', url).group(1)
	}


def load_config():
	with open(os.path.dirname(os.path.abspath(__file__)) + '/config.json', 'rb') as f:
		config = json.loads(f.read())
	for i, val in enumerate(config['exams']):
		if isinstance(val['lsf'], basestring):
			config['exams'][i]['lsf'] = url_to_params(val['lsf'])
		return config


def extract_registrations(config):
	session = requests.Session()

	# login
	session.get('https://www.lsf.uni-saarland.de/')  # get session id
	resp = session.post(
		'https://www.lsf.uni-saarland.de/qisserver/rds;jsessionid={}?state=user&type=1&category=auth.login&startpage=portal.vm&breadCrumbSource=portal'.format(
			session.cookies['JSESSIONID']),
		data={'username': config['lsf']['username'], 'password': config['lsf']['password'], 'submit': 'Ok'}
	)
	assert resp.status_code == 200
	if not re.search(r'asi=([A-Za-z0-9._\-,$]+)"', resp.text):
		with open('test.html', 'wb') as f:
			f.write(resp.text.encode('utf-8'))
	asi = re.search(r'asi=([A-Za-z0-9._\-,$]+)"', resp.text).group(1)
	assert asi

	results = []
	for exam in config['exams']:
		params = {
			'state': 'notenVerbuchungBearb',
			'struct': 'auswahlBaum',
			'next': 'teilnehmer.vm',
			'nextdir': 'qispos/prfListBearbeiter/notenverbuchung',
			'nodeID': 'auswahlBaum|termin:semester={},termin={}|pruefung:porgnr={},prueferID={}'.format(
				exam['lsf']['semester'], exam['lsf']['termin'], exam['lsf']['porgnr'], exam['lsf']['prueferID']),
			'expand': '0',
			'view': 'y',
			'lastState': 'notenVerbuchungBearb',
			'asi': asi
		}
		resp = session.get('https://www.lsf.uni-saarland.de/qisserver/rds', params=params)
		matriculations = re.findall(r'(\d{6,7})\s*</td>', resp.text)
		print(json.dumps(matriculations))
		results.append(matriculations)
	return results


def import_to_cms(cms_config, matriculations):
	cms = cakecms.CakeCMS(cms_config['url'], cms_config['token'], cms_config['course'])
	cms.assert_success(cms.registrations_import(cms_config['registration_id'], matriculations))
	print('Pushed {}#{}'.format(cms_config['course'], cms_config['registration_id']))


if __name__ == '__main__':
	config = load_config()
	if DEBUG:
		if os.path.exists('matriculations.json'):
			print('Reading matriculations from file...')
			with open('matriculations.json', 'rb') as f:
				matriculations = json.loads(f.read())
		else:
			matriculations = extract_registrations(config)
			with open('matriculations.json', 'wb') as f:
				f.write(json.dumps(matriculations))
	else:
		matriculations = extract_registrations(config)
	for i, exam in enumerate(config['exams']):
		import_to_cms(exam['cms'], matriculations[i])
	print('Done')
